import Foundation
import AppKit

private let supportedImageExtensions: [String : NSBitmapImageRep.FileType] = [
    "jpg" : .jpeg,
    "jpeg" : .jpeg,
    "png" : .png,
]

private let supportedVideoExtensions: Set<String> = [
    "mp4", "mov", "m4v"
]

enum MediaType {
    case image
    case video
    case file
    case svg

    init(fileExtension: String) {
        if supportedImageExtensions[fileExtension] != nil {
            self = .image
        } else if supportedVideoExtensions.contains(fileExtension) {
            self = .video
        } else if fileExtension == "svg" {
            self = .svg
        } else {
            self = .file
        }
    }

    static func isProcessableImage(_ fileExtension: String) -> Bool {
        supportedImage(fileExtension) != nil
    }

    static func supportedImage(_ fileExtension: String) -> NSBitmapImageRep.FileType? {
        supportedImageExtensions[fileExtension]
    }
}
